@@ -1,6 +1,6 @@
 use off_rs::{
-    geometry::{color_format::ColorFormat, mesh::Mesh},
-    parser::options::Options,
+    geometry::mesh::Mesh,
+    parser::{color_format::ColorFormat, options::Options},
     Error, Parse,
 };
 
